package com.aliyun.chatbot.channel;

import java.util.UUID;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.alibaba.fastjson2.JSONWriter.Feature;

import com.aliyun.chatbot.channel.base.ChannelMsgSender;
import com.aliyun.chatbot.channel.chatbot.ChatbotClient;
import com.aliyun.chatbot.channel.chatbot.ChatbotSSERequest;
import com.aliyun.chatbot.channel.dingtalk.DingtalkAccessTokenClient;
import com.aliyun.chatbot.channel.dingtalk.DingtalkChannelMsgSender;
import com.aliyun.chatbot.channel.dingtalk.DingtalkChatContext;
import com.aliyun.chatbot.channel.dingtalk.DingtalkInputMessage;
import com.aliyun.chatbot.channel.dingtalk.DingtalkInputMessage.TextContent;
import com.dingtalk.open.app.api.OpenDingTalkClient;
import com.dingtalk.open.app.api.OpenDingTalkStreamClientBuilder;
import com.dingtalk.open.app.api.callback.OpenDingTalkCallbackListener;
import com.dingtalk.open.app.api.security.AuthClientCredential;
import lombok.extern.slf4j.Slf4j;

@Slf4j
public class DingtalkRobotChatApp {

    // 机器人id
    public static final String CHATBOT_INSTANCE_ID = "chatbot-cn-******";

    // 业务空间AgentKey，获取地址：https://alime.console.aliyun.com/?productCode=p_ics#/business_management/business_management
    public static final String CHATBOT_AGENT_KEY = "**************_p_beebot_public";

    //aliyun账号AK、SK，登录阿里云账号获取
    public static final String ALIYUN_ACCESS_KEY = "<aliyun AK>";
    public static final String ALIYUN_SECRET_KEY = "<aliyun SK>";

    // 钉钉应用的 Client ID
    public static final String DINGTALK_CLIENT_ID = "dingqeqc**********";
    // 钉钉应用的 Client Secret
    public static final String DINGTALK_CLIENT_SECRET = "Etil**********dpR7v******S";
    // 钉钉AI卡片模板id
    public static final String DINGTALK_CARD_TEMP_ID = "********-****-****-****-*********.schema";

    public static void main(String[] args) throws Exception {
        ChatbotClient chatbotClient = new ChatbotClient(CHATBOT_AGENT_KEY, ALIYUN_ACCESS_KEY, ALIYUN_SECRET_KEY);

        DingtalkAccessTokenClient dingAkClient = new DingtalkAccessTokenClient(DINGTALK_CLIENT_ID, DINGTALK_CLIENT_SECRET);
        OpenDingTalkClient openDingTalkClient = OpenDingTalkStreamClientBuilder
            .custom()
            .credential(new AuthClientCredential(DINGTALK_CLIENT_ID, DINGTALK_CLIENT_SECRET))
            .registerCallbackListener("/v1.0/im/bot/messages/get",
                new RobotMsgCallbackConsumer(chatbotClient, dingAkClient))
            .build();

        while (true) {
            openDingTalkClient.start();
        }
    }

    public static class RobotMsgCallbackConsumer implements OpenDingTalkCallbackListener<JSONObject, JSONObject> {
        private static final long serialVersionUID = -1L;
        private final ChatbotClient chatbotClient;
        private final DingtalkAccessTokenClient dingAkClient;

        public RobotMsgCallbackConsumer(ChatbotClient chatbotClient,
            DingtalkAccessTokenClient dingAkClient) {
            this.chatbotClient = chatbotClient;
            this.dingAkClient = dingAkClient;
        }

        @Override
        public JSONObject execute(JSONObject request) {
            log.info("receive message by robot:{}", JSON.toJSONString(request, Feature.PrettyFormat));
            try {
                DingtalkInputMessage dingtalkInputMessage = request.toJavaObject(DingtalkInputMessage.class);
                TextContent text = dingtalkInputMessage.getText();
                if (text != null) {
                    //机器人接收消息内容
                    DingtalkChatContext context = new DingtalkChatContext()
                        .setRobotCode(dingtalkInputMessage.getRobotCode())
                        .setOutTrackId(UUID.randomUUID().toString())
                        .setDingtalkAccessTokenClient(dingAkClient)
                        .setDingtalkInputMessage(dingtalkInputMessage);
                    ChannelMsgSender sender = new DingtalkChannelMsgSender(context, DINGTALK_CARD_TEMP_ID);
                    sender.preSengMsg();

                    // 调用大模型
                    ChatbotSSERequest reqData = ChatbotSSERequest.build(CHATBOT_INSTANCE_ID,
                        dingtalkInputMessage.getText().getContent(),
                        dingtalkInputMessage.getConversationId());
                    chatbotClient.invokeLlm(reqData, sender);
                }
            } catch (Exception e) {
                log.error("receive message by robot error:" + e.getMessage(), e);
            }
            return new JSONObject();
        }
    }

}