package com.aliyun.chatbot.channel.dingtalk;

import java.util.concurrent.TimeUnit;

import com.alibaba.fastjson2.JSON;

import com.aliyun.dingtalkoauth2_1_0.Client;
import com.aliyun.dingtalkoauth2_1_0.models.GetAccessTokenRequest;
import com.aliyun.dingtalkoauth2_1_0.models.GetAccessTokenResponse;
import com.google.common.cache.CacheBuilder;
import com.google.common.cache.CacheLoader;
import com.google.common.cache.LoadingCache;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;

/**
 * @author kelin
 * @date 2024/6/4
 */
@Slf4j
public class DingtalkAccessTokenClient {
    private final String appKey;
    private final String appSecret;

    private static final LoadingCache<DingtalkAKRequest, String> CACHE = CacheBuilder.newBuilder()
        .maximumSize(10)
        .expireAfterWrite(90, TimeUnit.MINUTES)
        .build(new CacheLoader<DingtalkAKRequest, String>() {
            @Override
            public String load(DingtalkAKRequest key) throws Exception {
                return getAccessToken(key);
            }
        });

    public DingtalkAccessTokenClient(String appKey, String appSecret) {
        this.appKey = appKey;
        this.appSecret = appSecret;
    }

    private static Client createClient() throws Exception {
        com.aliyun.teaopenapi.models.Config config = new com.aliyun.teaopenapi.models.Config();
        config.protocol = "https";
        config.regionId = "central";
        return new Client(config);
    }

    public String getAccessToken() throws Exception {
        return CACHE.get(new DingtalkAKRequest(appKey, appSecret));
    }

    /**
     * 获取企业内部应用的accessToken
     * https://open.dingtalk.com/document/orgapp/obtain-the-access_token-of-an-internal-app
     */
    private static String getAccessToken(DingtalkAKRequest key) throws Exception {
        Client client = DingtalkAccessTokenClient.createClient();
        GetAccessTokenRequest getAccessTokenRequest = new GetAccessTokenRequest()
            .setAppKey(key.getAppKey())
            .setAppSecret(key.getAppSecret());
        GetAccessTokenResponse accessToken = client.getAccessToken(getAccessTokenRequest);
        log.info("dingtalk accessToken:{}", JSON.toJSONString(accessToken));
        return accessToken.getBody().getAccessToken();

    }

    @Data
    public static class DingtalkAKRequest {
        private final String appKey;
        private final String appSecret;
    }
}
